@@ -1,15 +1,18 @@
-#!/usr/bin/env python
 # -*- coding: utf-8 -*-
 import os
 import subprocess
 import time
 from abc import ABCMeta, abstractmethod
 
+import numpy as np
+
 import py2dmat
+import py2dmat.util.read_matrix
+import py2dmat.util.mapping
 
 # type hints
 from pathlib import Path
-from typing import List, Union, Optional
+from typing import List, Optional
 from . import mpi
 
 
@@ -84,22 +87,19 @@ class Logger:
                 f.write(f"# ${i}-: input\n")
             f.write("\n")
 
-
     def count(self, message: py2dmat.Message, result: float) -> None:
         if self.disabled():
             return
         self.num_calls += 1
         t = time.perf_counter()
-        fields = [self.num_calls, t-self.time_previous, t-self.time_start]
+        fields = [self.num_calls, t - self.time_previous, t - self.time_start]
         if self.to_write_result:
             fields.append(result)
         if self.to_write_x:
             for x in message.x:
                 fields.append(x)
         fields.append("\n")
-        self.buffer[
-            self.buffer_index
-        ] = " ".join(map(str, fields))
+        self.buffer[self.buffer_index] = " ".join(map(str, fields))
         self.time_previous = t
         self.buffer_index += 1
         if self.buffer_index == self.buffer_size:
@@ -116,15 +116,13 @@ class Logger:
 
 class Runner(object):
     solver: "py2dmat.solver.SolverBase"
-    run: Run
     logger: Logger
 
     def __init__(
         self,
         solver: py2dmat.solver.SolverBase,
         info: Optional[py2dmat.Info] = None,
-        nprocs: int = 1,
-        nthreads: int = 1,
+        mapping=None,
     ):
         """
 
@@ -135,39 +133,35 @@ class Runner(object):
         self.solver = solver
         self.solver_name = solver.name
         self.logger = Logger(info)
-        run_scheme = solver.default_run_scheme()
-        if run_scheme == "mpi_spawn_ready":
-            self.run = run_mpispawn_ready(
-                nprocs=nprocs,
-                nthreads=nthreads,
-                comm=mpi.comm(),
+
+        if mapping is None:
+            info_mapping = info.runner.get("mapping", {})
+            A: Optional[np.ndarray] = py2dmat.util.read_matrix.read_matrix(
+                info_mapping.get("A", [])
             )
-        elif run_scheme == "mpi_spawn":
-            self.run = run_mpispawn(
-                nprocs=nprocs,
-                nthreads=nthreads,
-                comm=mpi.comm(),
+            b: Optional[np.ndarray] = py2dmat.util.read_matrix.read_vector(
+                info_mapping.get("b", [])
             )
-        elif run_scheme == "subprocess":
-            self.run = run_subprocess(
-                nprocs=nprocs,
-                nthreads=nthreads,
-                comm=mpi.comm(),
-            )
-        elif run_scheme == "function":
-            self.run = run_function(comm=mpi.comm())
+            if A.size == 0:
+                A = None
+            if b.size == 0:
+                b = None
+            self.mapping = py2dmat.util.mapping.Affine(A=A, b=b)
         else:
-            msg = f"Unknown scheme: {run_scheme}"
-            raise ValueError(msg)
+            self.mapping = mapping
 
     def prepare(self, proc_dir: Path):
         self.logger.prepare(proc_dir)
 
-    def submit(self, message: py2dmat.Message) -> float:
-        self.solver.prepare(message)
+    def submit(
+        self, message: py2dmat.Message, nprocs: int = 1, nthreads: int = 1
+    ) -> float:
+        x = self.mapping(message.x)
+        message_indeed = py2dmat.Message(x, message.step, message.set)
+        self.solver.prepare(message_indeed)
         cwd = os.getcwd()
         os.chdir(self.solver.work_dir)
-        self.run.submit(self.solver)
+        self.solver.run(nprocs, nthreads)
         os.chdir(cwd)
         result = self.solver.get_results()
         self.logger.count(message, result)
@@ -175,52 +169,3 @@ class Runner(object):
 
     def post(self) -> None:
         self.logger.write()
-
-
-class run_mpispawn(Run):
-    def submit(self, solver: py2dmat.solver.SolverBase):
-        raise NotImplementedError()
-
-
-class run_mpispawn_ready(Run):
-    def submit(self, solver: py2dmat.solver.SolverBase):
-        raise NotImplementedError()
-
-
-class run_subprocess(Run):
-    """
-    Invoker via subprocess
-
-    """
-
-    def submit(self, solver: py2dmat.solver.SolverBase):
-        """
-        Run solver
-
-        Parameters
-        ----------
-        solver : py2dmat.solver.SolverBase
-            solver
-
-        Returns
-        -------
-        status : int
-            Always returns 0
-
-        Raises
-        ------
-        subprocess.CalledProcessError
-            Raises when solver failed.
-        """
-        command: List[str] = solver.command()
-        with open("stdout", "w") as fi:
-            subprocess.run(
-                command, stdout=fi, stderr=subprocess.STDOUT, check=True, shell=True
-            )
-        return 0
-
-
-class run_function(Run):
-    def submit(self, solver):
-        solver.function()()
-        return 0
