@@ -3,6 +3,7 @@ import itertools
 import os
 import os.path
 import shutil
+import subprocess
 from pathlib import Path
 
 import numpy as np
@@ -14,8 +15,6 @@ from py2dmat import exception
 class Solver(py2dmat.solver.SolverBase):
     path_to_solver: Path
 
-    dimension: int
-
     def __init__(self, info: py2dmat.Info):
         super().__init__(info)
 
@@ -31,30 +30,25 @@ class Solver(py2dmat.solver.SolverBase):
                     break
         if not os.access(self.path_to_solver, mode=os.X_OK):
             raise exception.InputError(f"ERROR: solver ({p2solver}) is not found")
+        info_config = info.solver.get("config", {})
 
         self.input = Solver.Input(info)
         self.output = Solver.Output(info)
-
-    def default_run_scheme(self):
-        """
-        Return
-        -------
-        str
-            run_scheme.
-        """
-        return "subprocess"
-
-    def command(self) -> List[str]:
-        """Command to invoke solver"""
-        return [str(self.path_to_solver)]
+        self.result = None
 
     def prepare(self, message: py2dmat.Message) -> None:
         fitted_x_list, subdir = self.input.prepare(message)
         self.work_dir = self.proc_dir / Path(subdir)
         self.output.prepare(fitted_x_list)
+        self.result = None
+
+    def run(self, nprocs: int = 1, nthreads: int = 1) -> None:
+        self._run_by_subprocess([str(self.path_to_solver)])
 
     def get_results(self) -> float:
-        return self.output.get_results(self.work_dir)
+        if self.result is None:
+            self.result = self.output.get_results(self.work_dir)
+        return self.result
 
     class Input(object):
         root_dir: Path
@@ -66,10 +60,14 @@ class Solver(py2dmat.solver.SolverBase):
         surface_template_file: Path
 
         def __init__(self, info):
-            self.dimension = info.base["dimension"]
             self.root_dir = info.base["root_dir"]
             self.output_dir = info.base["output_dir"]
 
+            if "dimension" in info.solver:
+                self.dimension = info.solver["dimension"]
+            else:
+                self.dimension = info.base["dimension"]
+
             info_s = info.solver
 
             info_param = info_s.get("param", {})
@@ -93,6 +91,8 @@ class Solver(py2dmat.solver.SolverBase):
                     f"ERROR: surface_template_file ({self.surface_template_file}) does not exist"
                 )
 
+            self._check_template()
+
             filename = info_config.get("bulk_output_file", "bulkP.b")
             filename = Path(filename).expanduser().resolve()
             self.bulk_output_file = self.root_dir / filename
@@ -104,7 +104,7 @@ class Solver(py2dmat.solver.SolverBase):
         def prepare(self, message: py2dmat.Message):
             x_list = message.x
             step = message.step
-            extra = message.set > 0
+            iset = message.set
 
             dimension = self.dimension
             string_list = self.string_list
@@ -118,7 +118,7 @@ class Solver(py2dmat.solver.SolverBase):
                 fitted_x_list.append(fitted_value)
             for index in range(dimension):
                 print(string_list[index], "=", fitted_x_list[index])
-            folder_name = self._pre_bulk(step, bulk_output_file, extra)
+            folder_name = self._pre_bulk(step, bulk_output_file, iset)
             self._replace(fitted_x_list, folder_name)
             return fitted_x_list, folder_name
 
@@ -135,11 +135,23 @@ class Solver(py2dmat.solver.SolverBase):
                             )
                     file_output.write(line)
 
-        def _pre_bulk(self, Log_number, bulk_output_file, extra):
-            if extra:
-                folder_name = "Extra_Log{:08d}".format(Log_number)
-            else:
-                folder_name = "Log{:08d}".format(Log_number)
+        def _check_template(self) -> None:
+            found = [False] * self.dimension
+            with open(self.surface_template_file, "r") as file_input:
+                for line in file_input:
+                    for index, placeholder in enumerate(self.string_list):
+                        if line.find(placeholder) != -1:
+                            found[index] = True
+            if not np.all(found):
+                msg = "ERROR: the following labels do not appear in the template file:"
+                for label, f in zip(self.string_list, found):
+                    if not f:
+                        msg += "\n"
+                        msg += label
+                raise exception.InputError(msg)
+
+        def _pre_bulk(self, Log_number, bulk_output_file, iset):
+            folder_name = "Log{:08d}_{:08d}".format(Log_number, iset)
             os.makedirs(folder_name, exist_ok=True)
             shutil.copy(
                 bulk_output_file, os.path.join(folder_name, bulk_output_file.name)
@@ -168,7 +180,11 @@ class Solver(py2dmat.solver.SolverBase):
         degree_list: List[float]
 
         def __init__(self, info):
-            self.dimension = info.base["dimension"]
+            if "dimension" in info.solver:
+                self.dimension = info.solver["dimension"]
+            else:
+                self.dimension = info.base["dimension"]
+
             info_s = info.solver
 
             # solver.config
@@ -216,6 +232,8 @@ class Solver(py2dmat.solver.SolverBase):
                 raise exception.InputError("ERROR: omega should be positive")
             self.omega = v
 
+            self.remove_work_dir = info_post.get("remove_work_dir", False)
+
             # solver.param
             info_param = info_s.get("param", {})
             v = info_param.setdefault("string_list", ["value_01", "value_02"])
@@ -285,6 +303,12 @@ class Solver(py2dmat.solver.SolverBase):
             os.chdir(work_dir)
             Rfactor = self._post(self.fitted_x_list)
             os.chdir(cwd)
+            if self.remove_work_dir:
+
+                def rmtree_error_handler(function, path, excinfo):
+                    print(f"WARNING: Failed to remove a working directory, {path}")
+
+                shutil.rmtree(work_dir, onerror=rmtree_error_handler)
             return Rfactor
 
         def _post(self, fitted_x_list):
